--
-- Loudmouth
-- Author: FatherJim of Blackhand
-- Email:  praetor@slayernet.org
--

--
-- Set up basic ACE2 library and Fubar data structures and methods.
--

local L = AceLibrary("AceLocale-2.2"):new("Loudmouth")
local tablet = AceLibrary("Tablet-2.0")
Loudmouth = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceDebug-2.0", "AceEvent-2.0", "FuBarPlugin-2.0")
Loudmouth:RegisterDB("LoudmouthDB","LoudmouthDBPC")
local crayon = AceLibrary("Crayon-2.0")

--Loudmouth.date="01/09/2007"
Loudmouth.version=0.805

--
-- Set up the shared SavedVariables.lua defaults.
--

Loudmouth:RegisterDefaults("char", {
  Version = 0, -- Loudmouth.version
  quotes = {
      ["AFK"] = {
		},
		["Crits"] = {
		},
		["Excuses"] = {
		},
		["Fears"] = {
		},
		["Misses"] = {
			[1] = {
				["emote"] = 1,
				["quote"] = "trips and falls.",
			},
		},
	},
})

Loudmouth:RegisterDefaults("profile", {
  --
  -- Default option values
  --
  Channel = 'Say',
  Debug = false,
  Throttle = 15,
  --
  -- Current State Information
  --
  Version = 0, -- Loudmouth.version
  ActiveSet = "Default",
  _QuoteSet = 1,
  _QuoteEvent = 1,
  enabled = {
    ["AFK"] = true,
    ["Crits"] = true,
    ["Excuses"] = true,
	 ["Fears"] = true,
	 ["Misses"] = true,
  },
  rates = {
    ["Crits"] = 15,
    ["Fears"] = 33,
    ["Misses"] = 50,
  },
  --
  -- Quotes and options
  --
  quotes = {
    ["Default"] = {
      ["AFK"] = {
        	--[1] = {
				--["emote"] = 0,
				--["quote"] = "Oops, my ethernet cable fell out, and the bits are making a mess on the carpet... brb",
			--},
			--[2] = {
				--["emote"] = 0,
				--["quote"] = "Got a phone call. I won the Lottery. Back after I buy Blizzard to fix a few things...",
			--},
      },
      ["Crits"] = {
        	--[1] = {
				--["emote"] = 0,
				--["quote"] = "I just got a critical hit!",
			--},
			--[2] = {
				--["emote"] = 0,
				--["quote"] = "Oops, my weapon just got stuck in you, could you hand that back please?",
			--},
      },
      ["Excuses"] = {
      	--[1] = {
				--["emote"] = 0,
				--["quote"] = "My g/f just called and wife picked up the phone. Sorry, in combat IRL!"
			--},
      },
      ["Fears"] = {
        	--[1] = {
				--["emote"] = 0,
				--["quote"] = "Hey mob! I didn't say you could leave yet...",
			--},
			--[2] = {
				--["emote"] = 0,
				--["quote"] = "Oh sure, fight me until you think you're gonna lose, then run away like a little girlie...",
			--},
      },
      ["Misses"] = {
        	--[1] = {
				--["emote"] = 0,
				--["quote"] = "Stand still or come closer so I can slap you!",
			--},
			--[2] = {
				--["emote"] = 0,
				--["quote"] = "Oops... I missed.",
			--},
      },
    },
  },
})

--
-- Set up the internal variables, menus and structures that aren't user data
--
local scrollbar = getglobal("LMFrame_QuoteList_ScrollFrame_ScrollBar")
local statement = nil
local currentquote = {
	["quote"] = "",
	["emote"] = 0,
}
local quotelist = { }

Loudmouth.dropdowns={}
Loudmouth.dropdowns.Lists={};
Loudmouth.dropdowns.Lists["_QuoteEvent"] = { index = "_QuoteEvent", label = "Event Type", selected = 2, table = { [1] = "AFK", [2] = "Crits", [3] = "Excuses", [4] = "Fears", [5] = "Misses" } }
Loudmouth.dropdowns.Lists["_QuoteSet"] = { index = "_QuoteSet", label = "Quote Set", selected = 2, table = {[1] = "Default", [2] = "Char"} }
Loudmouth.dropdowns.Lists["_SendQuote"] = { index = "_SendQuote", label = "", selected = 1, table = { [1] = "Send on Channel", [2] = "Say", [3] = "Yell", [4] = "Party", [5] = "Raid", [6] = "Guild", [7] = "Officer" } }
Loudmouth.editing_item=0
Loudmouth.hasIcon = true
Loudmouth.hideMenuTitle = true
Loudmouth.LastQuoted = 0
Loudmouth.lastQuoteID = {
	 ["AFK"] = 0,
    ["Crits"] = 0,
	 ["Excuses"] = 0,
	 ["Fears"] = 0,
	 ["Misses"] = 0,
  }

local options = {
    type = 'group',
    args = {
    	header = {
	    type = 'header',
	    name = L["Loudmouth"],
	    icon = "Interface\\AddOns\\Loudmouth\\icon.tga",
	    iconHeight = 16,
	    iconWidth = 16,
	    order = 1,
	},
        --msg = {
	    --type = 'text',
	    --name = L["Message"],
	    --desc = L["Sets the message to be displayed when you enter a new zone."],
	    --usage = L["<your message>"],
	    --get = "GetMessage",
	    --set = "SetMessage",
	--},
	afks = {
		type = 'toggle',
		name = L["AFK Messages"],
		desc = L["Whether AFKs messages will be sent"],
		get = function ()
				return Loudmouth:Get_Event_State("AFK")
		end,
		set = function ()
				Loudmouth:Set_Event_State("AFK", not Loudmouth:Get_Event_State("AFK"))
				if (LMFrame:IsShown()) then Loudmouth:GUI_Rate_Show() end
	    		Loudmouth:Update()
		end,
	},
	crits = {
	    type = 'toggle',
	    name = L["Crit Hits"],
	    desc = L["Will you brag about critical hits?"],
		get = function ()
				return Loudmouth:Get_Event_State("Crits")
		end,
		set = function ()
				Loudmouth:Set_Event_State("Crits", not Loudmouth:Get_Event_State("Crits"))
				if (LMFrame:IsShown()) then Loudmouth:GUI_Rate_Show() end
	    		Loudmouth:Update()
	    end,
	},
	critrate = {
	    type = 'range',
	    name = L["Crit Rate"],
	    desc = L["How often you brag about critical hits"],
	    min = 0,
	    max = 100,
	    step = 1,
	    isPercent = false,
	    get = function ()
	        	return Loudmouth:Get_Rate("Crits")
	    end,
	    set = function (v)
	         Loudmouth:Set_Rate("Crits",v)
				if (LMFrame:IsShown()) then Loudmouth:GUI_Rate_Show() end
	    		Loudmouth:Update()
	    end,
	},
	excuses = {
		type = 'toggle',
		name = L["Excuse Messages"],
		desc = L["Whether Excuse messages will be sent"],
		get = function ()
				return Loudmouth:Get_Event_State("Excuses")
		end,
		set = function ()
				Loudmouth:Set_Event_State("Excuses", not Loudmouth:Get_Event_State("Excuses"))
				if (LMFrame:IsShown()) then Loudmouth:GUI_Rate_Show() end
	    		Loudmouth:Update()
		end,
	},
	fears = {
	    type = 'toggle',
	    name = L["Fear Effects"],
	    desc = L["Will you comment when mobs run away?"],
		get = function ()
				return Loudmouth:Get_Event_State("Fears")
		end,
		set = function ()
				Loudmouth:Set_Event_State("Fears", not Loudmouth:Get_Event_State("Fears"))
				if (LMFrame:IsShown()) then Loudmouth:GUI_Rate_Show() end
	    		Loudmouth:Update()
	    end,
	},
	fearrate = {
	    type = 'range',
	    name = L["Fear Rate"],
	    desc = L["How often you taunt about Feared mobs running away"],
	    min = 0,
	    max = 100,
	    step = 1,
	    isPercent = false,
	    get = function ()
	        return Loudmouth:Get_Rate("Fears")
	    end,
	    set = function (v)
	         Loudmouth:Set_Rate("Fears",v)
				if (LMFrame:IsShown()) then Loudmouth:GUI_Rate_Show() end
	    		Loudmouth:Update()
	    end,
	},
	miss = {
	    type = 'toggle',
	    name = L["Missed Attacks"],
	    desc = L["Will you comment if you miss?"],
		get = function ()
				return Loudmouth:Get_Event_State("Misses")
		end,
		set = function ()
				Loudmouth:Set_Event_State("Misses", not Loudmouth:Get_Event_State("Misses"))
				if (LMFrame:IsShown()) then Loudmouth:GUI_Rate_Show() end
	    		Loudmouth:Update()
	    end,
	},
	missrate = {
	    type = 'range',
	    name = L["Missed Attack Rate"],
	    desc = L["How often you taunt about Missed Attacks"],
	    min = 0,
	    max = 100,
	    step = 1,
	    isPercent = false,
	    get = function ()
	        	return Loudmouth:Get_Rate("Misses")
	    end,
	    set = function (v)
	         Loudmouth:Set_Rate("Misses",v)
				if (LMFrame:IsShown()) then Loudmouth:GUI_Rate_Show() end
	    		Loudmouth:Update()
	    end,
	},
	channel = {
	    type = 'text',
	    name = L["Babble Channel"],
	    desc = L["Babble randomly in selected channel"],
	    get = function()
	        	return Loudmouth.db.profile.Channel
	    end,
	    set = function(v)
	        	Loudmouth.db.profile.Channel = v
	    end,
	    validate = { "UI", "Say", "Party", "Raid", "Guild", "Officer" },
	},
	throttle = {
	    type = 'range',
	    name = L["Wisecrack Cooldown"],
	    desc = L["Minimum time between Wisecracks."],
	    min = 5,
	    max = 300,
	    step = 5,
	    isPercent = false,
	    get = function ()
	        	return Loudmouth.db.profile.Throttle
	    end,
	    set = function (v)
	         Loudmouth.db.profile.Throttle = v
	    		Loudmouth:Update()
	    end,
	},
	afk = {
		type = 'execute',
		name = L["Show AFK Message"],
		desc = L["Show an AFK Message in Guild chat, and raid or party"],
		func = function()
		  		Loudmouth:Output_AFK()
		end,
	},
	excuse = {
		type = 'execute',
		name = L["Show Excuse"],
		desc = L["Show an excuse to Guild and default babble channel"],
		func = function()
		  		Loudmouth:Output_Excuse()
		end,
	},
	gui = {
		type = 'execute',
		name = L["Show GUI"],
		desc = L["Display GUI Window"],
		func = function()
		  		Loudmouth:GUI_Show()
		end,
	},
	--hide = {
	   --type = 'execute',
	   --name = 'Hide GUI',
	   --desc = 'Close the GUI Frame',
	   --func = "Loudmouth_Frame:Hide()",
	--},
	state = {
	   type = 'execute',
	   name = L["Show Status"],
	   desc = L["Show Command Line Status"],
		func = function()
		 		Loudmouth:Output_Command_Line_Status()
		end,
	},
  },
}

--
-- Set up ACE and Fubar menus
--

Loudmouth:RegisterChatCommand({"/loudmouth","/lm"}, options)
Loudmouth.OnMenuRequest = options

--
-- Ace / Fubar Interface handlers
--

-- If you Left Click on FUBAR icon/text...
function Loudmouth:OnClick()
  Loudmouth:GUI_Show() 
end

--function Loudmouth:OnMenuRequest()
  --self:AddImpliedMenuOptions()
--end

-- Let's build a nice tooltip showing the current state...
function Loudmouth:OnTooltipUpdate()
  local WisecrackCooldown
  local cat = tablet:AddCategory(
    'text', L["Event Type"], 'text2', L["Response Rates"],
	 'columns', 2,
    'child_textR' , 1, 'child_textG' , 1, 'child_textB' , 0,
    'child_textR2', 1, 'child_textG2', 1, 'child_textB2', 0
  )
  if (self:Get_Event_State("AFK")) then
    cat:AddLine( 'text', L["AFK Messages"], 'text2', self:Get_Rate("AFK"))
  end
  if (self:Get_Event_State("Crits")) then
    cat:AddLine( 'text', L["Crit Hits"], 'text2', self:Get_Rate("Crits"))
  end
  if (self:Get_Event_State("Excuses")) then
    cat:AddLine( 'text', L["Excuse Messages"], 'text2', self:Get_Rate("Excuses"))
  end
  if (self:Get_Event_State("Fears")) then
    cat:AddLine( 'text', L["Fear Effects"], 'text2', self:Get_Rate("Fears"))
  end
  if (self:Get_Event_State("Misses")) then
    cat:AddLine( 'text', L["Missed Attacks"], 'text2', self:Get_Rate("Misses"))
  end
  local cat = tablet:AddCategory(
    'text', L["Configuration"], 'columns', 2,
    'child_textR' , 1, 'child_textG' , 1, 'child_textB' , 0,
    'child_textR2', 0, 'child_textG2', 1, 'child_textB2', 1
  )
  cat:AddLine( 'text', L["Babble Channel"], 'text2', self.db.profile.Channel)
  cat:AddLine( 'text', L["Number of Quotes"], 'text2', self.current_total_quotes)
  local activeset
  if (self.current_set_name == "Char") then
    activeset = self.db.profile.ActiveSet
  else
    activeset = self.current_set_name
  end
  cat:AddLine( 'text', L["Quote Set in Use"], 'text2', activeset)
  if (self:IsDebugging() == true) then
    WisecrackCooldown = L["Disabled"]
  else
    WisecrackCooldown = self.db.profile.Throttle..L["seconds"]
  end
  cat:AddLine( 'text', L["Wisecrack Cooldown"], 'text2', WisecrackCooldown)

  --local hours,minutes = GetGameTime();
  --cat:AddLine( 'text', L["Game Time"], 'text2', hours..":"..minutes)
  --cat:AddLine( 'text', L["System Uptime"], 'text2', GetTime())
  tablet:SetTitle(L["Loudmouth"])
  tablet:SetHint(L["Left click to bring up the configuration window. Right click to edit options."])
  self.db.profile.Debug = self:IsDebugging()
end

function Loudmouth:Output_Command_Line_Status()
  local eventTypes = ""
  local responseTypes = ""
  local channelName = ""

  for k,v in pairs(self.db.profile.enabled) do
    if (v)
	 then	eventTypes = eventTypes.." "..crayon:Colorize("00cf00",k)
	 else	eventTypes = eventTypes.." "..crayon:Colorize("ff0000",k)
	 end
  end
  for k,v in pairs(self.db.profile.rates) do
    --if (self:Get_Event_State(k) == true)
    if (self:Get_Event_State(k))
	 then
    responseTypes = responseTypes.." "..crayon:Colorize("00cf00",k).."="..crayon:Colorize("00cf00",v)
	 else
    responseTypes = responseTypes.." "..crayon:Colorize("ff0000",k).."="..crayon:Colorize("ff0000",v)
	 end
  end
  if (self.db.profile.Channel == "Raid")
  then channelName = crayon:Colorize("ff4f2f","Raid")
  elseif (self.db.profile.Channel == "Party")
  then channelName = crayon:Colorize("7f9fff","Party")
  elseif (self.db.profile.Channel == "Say")
  then channelName = crayon:Colorize("ffffff","Say")
  elseif (self.db.profile.Channel == "UI")
  then channelName = crayon:Colorize("ffffff","UI")
  elseif (self.db.profile.Channel == "Guild")
  then channelName = crayon:Colorize("00ff00","Guild")
  elseif (self.db.profile.Channel == "Officer")
  then channelName = crayon:Colorize("00bf00","Officer")
  end

  self:Print(L["Loudmouth"].." "..self.version.." "..L["status"]..":")
  self:Print("  "..L["Event Type"].."s:"..eventTypes)
  self:Print("  "..L["Response Rates"]..":"..responseTypes)
  self:Print("  "..L["Babble Channel"]..": "..channelName)
  self:Print("  "..L["Number of Quotes"]..": "..crayon:Colorize("9f9fff",self.current_total_quotes))
  self:Print("  "..L["Quote Set in Use"]..": "..crayon:Colorize("00bf00",self.current_set_name))
  self:Print("  "..L["Wisecrack Cooldown"]..": "..crayon:Colorize("9f9fff",self.db.profile.Throttle.." "..L["seconds"]))

  self.db.profile.Debug = self:IsDebugging()
end

--
-- ACE core functions
--

--function Loudmouth:OnLoad()
  --self:Debug(L["Loudmouth"].." loaded.")
--end

function Loudmouth:OnDisable()
  self:Print("Loudmouth Disabling...");
end

function Loudmouth:OnEnable()
  -- Called when the addon is enabled
  if (self.db.profile.Version ~= Loudmouth.version)
  then	self:OnUpgrade()
  			self.db.profile.Version = Loudmouth.version
  end
  LMFrame_QuoteList_Highlight_Texture:SetVertexColor(0.70, 0.70, 0.20)
  self:SetDebugging(Loudmouth.db.profile.Debug)
  self:Auto_Vars()
  self.LastQuoted = GetTime()
  self:RegisterMyEvents()
end

function Loudmouth:OnUpgrade()
  -- This section called if upgrading from 0.3
  if (self.db.profile.critrate ~= nil) then	
    self.db.profile.rates["Crits"]=self.db.profile.critrate
  	 table.remove(self.db.profile,critrate)
	 self:Print(L["Upgrading from 0.3"]);
  end
  if (self.db.profile.fearrate ~= nil) then	
    self.db.profile.rates["Fears"]=self.db.profile.fearrate
  	 table.remove(self.db.profile,fearrate)
  end
  if (self.db.profile.missrate ~= nil) then	
    self.db.profile.rates["Misses"]=self.db.profile.missrate
  	 table.remove(self.db.profile,missrate)
  end

  -- This section called if upgrading from < 0.6
  if (self.db.profile.crits) then	
    self.db.profile.enabled["Crits"] = self.db.profile.crits
  	 table.remove(self.db.profile,crits)
	 self:Print(L["Upgrading from below 0.6"]);
  end
  if (self.db.profile.fears) then	
    self.db.profile.enabled["Fears"] = self.db.profile.fears
  	 table.remove(self.db.profile,fears)
  end
  if (self.db.profile.miss) then	
    self.db.profile.enabled["Misses"] = self.db.profile.miss
  	 table.remove(self.db.profile,miss)
  end

  -- This section called if upgrading from < 0.8
  if (tonumber(self.db.profile.Version) < 0.8) then
    self:Print(L["Upgrading from below 0.8"])
    local set, event, index, value, value2, value3
    for set,value in pairs(self.db.profile.quotes) do
      self:Debug("Working in Quote Set: "..set)
		if (not self.db.profile.quoteopts) then
		  self.db.profile.quoteopts = {}
		end
		if (not self.db.profile.quoteopts[set]) then	
		  self.db.profile.quoteopts[set] = {}
		end
  	   for event,value2 in pairs(self.db.profile.quotes[set]) do
        self:Debug("Working in Quote Event: "..event)
		  if (not self.db.profile.quoteopts[set][event]) then	
		    self.db.profile.quoteopts[set][event] = {}
		  end
	 	  for index,value3 in pairs(self.db.profile.quotes[set][event]) do
          self:Debug("Working in Quote Index: "..index)
		    if (not self.db.profile.quoteopts[set][event][index]) then	
			   self.db.profile.quoteopts[set][event][index] = {}
		    end
		    if (not self.db.profile.quoteopts[set][event][index]["emote"]) then	
			   self.db.profile.quoteopts[set][event][index]["emote"] = 0
		    end
		  end -- for index
	   end -- for event
    end -- for set
  end -- upgrade to 0.8

  -- This section called if upgrading from < 0.804
  if (tonumber(self.db.profile.Version) < 0.804) then
    self:Print(L["Upgrading from below 0.804"])
	 local set, event, index, index2, thisquote, value, value2, value3
	 for set, value in pairs(self.db.profile.quotes) do
	   self:Debug("Working in Quote Set: "..set)
		for event, value2 in pairs(self.db.profile.quotes[set]) do
        self:Debug("Working in Quote Event: "..event)
		  index2 = 1
		  for index, value3 in pairs(self.db.profile.quotes[set][event]) do
          self:Debug("Working in Quote: "..index)
			 if (self.db.profile.quotes[set][event][index] ~= nil) then
			   if (not self.db.profile.quotes[set][event][index]["quote"]) then
		        thisquote = self.db.profile.quotes[set][event][index]
			     self.db.profile.quotes[set][event][index2] = {}
			     self.db.profile.quotes[set][event][index2]["quote"] = thisquote
			   end
			   if (not self.db.profile.quoteopts or not self.db.profile.quoteopts[set] or not self.db.profile.quoteopts[set][event] or not self.db.profile.quoteopts[set][event][index]) then
			     self.db.profile.quotes[set][event][index2]["emote"] = 0
			   else
			     self.db.profile.quotes[set][event][index2]["emote"] = self.db.profile.quoteopts[set][event][index]["emote"]
			   end
				index2 = index2 + 1
			 end -- index is nil
		  end -- for index
		  for index = index2, #self.db.profile.quotes[set][event] do
		    table.remove(self.db.profile.quotes[set][event],index)
		  end -- for index
		end -- for event
	 end -- for set
	 table.remove(self.db.profile,quoteopts)
  end -- upgrade to 0.804

  self:Print(L["Upgrading data structures from version "]..self.db.profile.Version..L[" to "]..Loudmouth.version)
end

--
-- Loudmouth UI code
--

function Loudmouth:GUI_Check_Click(box)
  if (box:GetName() == "LMEditFrame_Emote") then	
	 if (self.editing_item ~= 0) then
      self.db.profile.quotes[self.current_set_name][self.current_event_name][self.editing_item]["emote"]=box:GetChecked()
	 end
  end
  if (box:GetName() == "LMFrame_Enable") then
    self:Set_Event_State(self.current_event_name,box:GetChecked())
    self:GUI_Rate_Show()
  end
  self:Update()
end

function Loudmouth:GUI_DropDown_Initialize()
  local info, index, value, key;
  for index, value in pairs(Loudmouth.dropdowns.Lists) do
  	if (this:GetName() == "LMFrame"..value.index.."Button" or
  	    this:GetName() == "LMFrame"..value.index)
	then
		local label = getglobal("LMFrame"..index.."_Label")
		if (strlen(value.label) >= 2) then
			label:SetText(value.label)
		end
		for key = 1, #value.table do
			info = UIDropDownMenu_CreateInfo()
			if (self.db.profile)
			then
				if (key == self.db.profile[index])
				then	info.checked = true
				else	info.checked = nil
				end
			end
			info.text = value.table[key]
			info.func = function(x) self:GUI_DropDown_OnClick(x) end
			info.arg1 = value.index
			UIDropDownMenu_AddButton(info)
		end
		if (string.find(this:GetName(),"_SendQuote"))
		then	local name,number
				for k = 1, 10  do
				   number,name = GetChannelName(k)
					if (name ~= nil) then
						info = UIDropDownMenu_CreateInfo()
						info.text = name
						info.value = k
						info.func = function(x) self:GUI_DropDown_OnClick(x) end
						info.arg1 = value.index
						UIDropDownMenu_AddButton(info)
					end
				end
		end
		break;
	end
	UIDropDownMenu_SetSelectedID(this, this:GetID())
  end
end

function Loudmouth:GUI_DropDown_OnClick(list)
  local ddl = getglobal("LMFrame"..list)
  UIDropDownMenu_SetSelectedID(ddl, this:GetID())
  local selected = UIDropDownMenu_GetSelectedValue("LMFrame"..list)

  if (list == "_SendQuote")
  then	local channel = this:GetText()
  			if (this:GetID() ~= 1)
			then
				if (this:GetID() > #self.dropdowns.Lists["_SendQuote"].table)
				then
					SendChatMessage(LMFrame_QuoteEdit:GetText(),"CHANNEL",nil,GetChannelName(channel))
				else
  					SendChatMessage(LMFrame_QuoteEdit:GetText(),channel)
				end
    			UIDropDownMenu_ClearAll(LMFrame_SendQuote)
    			UIDropDownMenu_SetSelectedID(LMFrame_SendQuote,1)
			end
  			return
  end

  if (this:GetID() ~= self.db.profile[list]) then
   if (list == "_QuoteSet") then
	  if (this:GetText() ~= "Char") then
	    self.db.profile.ActiveSet = this:GetText()
	  end
	end
  	self.db.profile[list] = this:GetID()
	self:GUI_Input_Disable()
   LMFrame_QuoteEdit:ClearFocus()
	self:Auto_Vars()
	self:GUI_Rate_Show()
  	self:GUI_ListBox_OnShow("LMFrame"..list)

  else
    self.db.profile[list] = this:GetID()
	 self:GUI_Rate_Show()
  end
  self:Update()
end

function Loudmouth:GUI_DropDown_OnShow()
  local listname = string.sub(this:GetName(),8)

  if (listname == "_QuoteSet") then
    UIDropDownMenu_SetSelectedName(this,self.current_set_name)
    UIDropDownMenu_SetWidth(80,LMFrame_QuoteSet)
  end
  if (listname == "_QuoteEvent") then
  	 UIDropDownMenu_SetSelectedName(this,self.current_event_name)
    UIDropDownMenu_SetWidth(80,LMFrame_QuoteEvent)
	 --LMFrame_QuoteEvent:ClearAllPoints()
	 --LMFrame_QuoteEvent:SetPoint("LEFT",LMFrame_QuoteSet,"RIGHT", 5, 0)
  end
  if (listname == "_SendQuote") then
	 self:GUI_DropDown_Initialize(this)
    UIDropDownMenu_SetSelectedID(LMFrame_SendQuote,1)
  end
end

function Loudmouth:GUI_Input_Disable(keepfocus)
  if not keepfocus then
    keepfocus = 0
    --LMFrame_QuoteEdit:ClearFocus()
  --else
    --LMFrame_QuoteEdit:SetFocus()
  end

  --self:Print("InputDisable(Keepfocus) = "..keepfocus)
  --if (keepfocus ~= 1) then
    --LMFrame_QuoteEdit:ClearFocus()
  --end
  LMFrame_QuoteEdit:SetText("")
  LMFrame_QuoteList_Highlight:Hide()
  self.editing_item = 0
  LMFrame_ButtonDelete:Hide()
  LMFrame_ButtonCancel:Hide()
  LMFrame_ButtonSave:Hide()
  LMFrame_SendQuote:Hide()
  LMEditFrame_Emote:SetChecked(0)
  LMEditFrame:Hide()
end

function Loudmouth:GUI_Input_Enable()
  LMFrame_ButtonCancel:Show()
  LMFrame_ButtonSave:Show()
  if (self.editing_item ~= 0) then
    LMFrame_ButtonSave:SetText("Update")
	 LMFrame_ButtonCancel:SetText("Cancel Update")
    LMFrame_ButtonDelete:Show()
  else
    LMFrame_ButtonSave:SetText("Add")
	 LMFrame_ButtonCancel:SetText("Cancel Add")
  end
  LMEditFrame:Show()
  LMFrame_SendQuote:Show()
end

function Loudmouth:GUI_ListBox_OnScroll()
  self:Auto_Vars()
  local scrollbar = LMFrame_QuoteList_ScrollFrame_ScrollBar
  local buttonNo = 1
  local n = scrollbar:GetValue() - 1

  for i = 1, 15, 1 do
    getglobal("LMFrame_QuoteList_Button"..i.."_Text"):SetText("");
  end

  local temp = self.editing_item - n
  if temp <= 0 or temp >= 16 then
    LMFrame_QuoteList_Highlight:Hide();
  else
    local highlight = LMFrame_QuoteList_Highlight;
    highlight:Show();
    highlight:ClearAllPoints();
    highlight:SetPoint("TOPLEFT", "LMFrame_QuoteList_Button"..temp, "TOPLEFT", 0, 0);
  end

  --self:GUI_ListBox_OnShow()
  LMFrame_QuoteList_ScrollFrame_ScrollBar_ScrollUpButton:Disable()
  for key = 1, #quotelist do
    value = quotelist[key]["quote"]
    if n == 0 then
      if buttonNo == 16 then
        LMFrame_QuoteList_ScrollFrame_ScrollBar_ScrollDownButton:Enable();
		  return
      end
      getglobal("LMFrame_QuoteList_Button"..buttonNo.."_Text"):SetText(value);
      buttonNo = buttonNo + 1;
    else
      n = n - 1;
		LMFrame_QuoteList_ScrollFrame_ScrollBar_ScrollUpButton:Enable()
    end
  end
  LMFrame_QuoteList_ScrollFrame_ScrollBar_ScrollDownButton:Disable()
end

function Loudmouth:GUI_ListBox_OnShow(offset)
  self:Auto_Vars()
  local i = 4;
  local scrollsteps = self.current_num_quotes-14
  if (scrollsteps < 1) then scrollsteps = 1 end
  if (not offset) then
    offset = 1
  end

  LMFrame_QuoteList:EnableMouseWheel(true)
  LMFrame_QuoteList:SetScript("OnMouseWheel", function()
  	if arg1 > 0 then LMFrame_QuoteList_ScrollFrame_ScrollBar_ScrollUpButton:Click() 
	elseif arg1 < 0 then LMFrame_QuoteList_ScrollFrame_ScrollBar_ScrollDownButton:Click()
	end
  end)
  for i = 1, 15 do
    getglobal("LMFrame_QuoteList_Button"..i.."_Text"):SetText("")
    getglobal("LMFrame_QuoteList_Button"..i..""):Hide()
  end
  LMFrame_QuoteList_ScrollFrame:Hide()
  LMFrame_QuoteList_Label:SetText(L["Quote List"].." - "..self.current_num_quotes.." "..L["quotes"]);
  for i = 1, #quotelist
  do
	if (i <= 15)
	then	getglobal("LMFrame_QuoteList_Button"..i.."_Text"):SetText(quotelist[i]["quote"])
    		getglobal("LMFrame_QuoteList_Button"..i):Show()
	else	LMFrame_QuoteList_ScrollFrame:Show()
	end
  end
  LMFrame_QuoteList_ScrollFrame_ScrollBar:SetMinMaxValues(1,scrollsteps)
  LMFrame_QuoteList_ScrollFrame_ScrollBar:SetValue(1)
  LMFrame_QuoteList_ScrollFrame_ScrollBar_ScrollUpButton:Disable()
  if (scrollsteps > 1)
  then	LMFrame_QuoteList_ScrollFrame_ScrollBar_ScrollDownButton:Enable()
  else	LMFrame_QuoteList_ScrollFrame_ScrollBar_ScrollDownButton:Disable()
  end
end

function Loudmouth:GUI_ListButton_OnClick(button)
  self:Auto_Vars()
  local listnum=tonumber(strsub(button:GetName(),strfind(button:GetName(),"n",-3)+1))
  local quotenum=listnum+LMFrame_QuoteList_ScrollFrame_ScrollBar:GetValue()-1
  if (quotenum <= #quotelist)
  then
    self.editing_item=quotenum
    LMFrame_QuoteEdit:SetText(getglobal(button:GetName().."_Text"):GetText())
	 LMEditFrame_Emote:SetChecked(quotelist[self.editing_item]["emote"])
    LMFrame_QuoteList_Highlight:Show()
    LMFrame_QuoteList_Highlight:ClearAllPoints()
    LMFrame_QuoteList_Highlight:SetPoint("TOPLEFT",button:GetName(), "TOPLEFT", 0, 0)
	 LMFrame_QuoteEdit:SetFocus()
	 LMFrame_QuoteEdit:HighlightText(strlen(LMFrame_QuoteEdit:GetText()))
  else
    self:Print("Not editing that quote, it's blank.");
  end
end

function Loudmouth:GUI_ListButton_HideTooltip(button)
  GameTooltip:Hide()
end

function Loudmouth:GUI_ListButton_ShowTooltip(button)
  self:Auto_Vars()
  local boxwidth, textwidth
  local thisquote
  local listnum=tonumber(strsub(button:GetName(),strfind(button:GetName(),"n",-3)+1))
  local quotenum=listnum+LMFrame_QuoteList_ScrollFrame_ScrollBar:GetValue()-1

  LMFrame_Temp_String_Text:SetText(getglobal(this:GetName().."_Text"):GetText())
  boxwidth = getglobal(this:GetName().."_Text"):GetStringWidth()
  textwidth = LMFrame_Temp_String_Text:GetWidth()

  if (textwidth > boxwidth or self:Quote_Has_Options(quotenum) == true)
  then
      thisquote = getglobal(this:GetName().."_Text"):GetText()
		if (quotelist[quotenum]["emote"] == 1) then
		  thisquote = "/me "..thisquote
		end
  		GameTooltip:SetOwner(button,"ANCHOR_TOPLEFT")
  		GameTooltip:AddLine("Quote:",  1.0, 1.0, 1.0, 1.0, 0)
		GameTooltip:AddLine(thisquote, 0.3, 0.8, 1.0, 1.0, 1)
		if (self:Quote_Has_Options(quotenum)) then
  		  GameTooltip:AddLine("Options:",  1.0, 1.0, 1.0, 1.0, 0)
		end
		if (quotelist[quotenum]["emote"] == 1) then
		  GameTooltip:AddLine("Emote", 0.3, 0.8, 1.0, 1.0, 1)
		end
		GameTooltip:Show()
  end
end

function Loudmouth:GUI_Quote_Edit_Updating()
  if (strlen(LMFrame_QuoteEdit:GetText()) >= 1)
  then	self:GUI_Input_Enable()
  else	self:GUI_Input_Disable()
  end
end

function Loudmouth:GUI_Rate_Edit_Changed(Box)
  if (Box == nil) then return end
  self:Set_Rate(self.current_event_name, tonumber(Box:GetText()))
  getglobal(Box:GetParent():GetName().."_Slider"):SetValue(tonumber(Box:GetText()))
  self:Update()
end

function Loudmouth:GUI_Rate_Show()
  if (self.current_event_name == "Excuses" or
      self.current_event_name == "AFK")
  then	LMFrame_Rate:Hide()
  			LMFrame_Enable:Show()
			LMFrame_Enable:SetChecked(self:Get_Event_State())
  else	
			if (self:Get_Event_State())
			then	LMFrame_Rate:Show()
			else	LMFrame_Rate:Hide()
			end
  			LMFrame_Enable:Show()
  			LMFrame_Rate_Label:SetText(self.current_event_name..L[" Rate     Slider    Value  "])
			LMFrame_Enable:SetChecked(self:Get_Event_State())
  			LMFrame_Rate_EditBox:SetText(self:Get_Rate())
  			LMFrame_Rate_SliderLow:SetText("0")
  			LMFrame_Rate_SliderHigh:SetText("100")
  			LMFrame_Rate_Slider:SetMinMaxValues(0,100)
  			LMFrame_Rate_Slider:SetValueStep(1)
  			LMFrame_Rate_Slider:SetValue(self:Get_Rate())
  end
end

function Loudmouth:GUI_Rate_Slider_Changed(Slider)
  if (Slider == nil) then return end
  self:Set_Rate(self.current_event_name, tonumber(Slider:GetValue()))
  getglobal(Slider:GetParent():GetName().."_EditBox"):SetText(tonumber(Slider:GetValue()))
  self:Update()
end

function Loudmouth:GUI_Show()
  self.db.profile.Debug = self:IsDebugging()
  LMFrame_Title:SetText(L["Loudmouth"]);
  LMFrame_QuoteList_Label:SetText(L["Quote List"]);
  LMFrame_Version:SetText(L["version"].." "..Loudmouth.version.." "..Loudmouth.date)
  LMFrame:Show();
  self:Auto_Vars()
  self:GUI_Rate_Show()
end

--
-- Loudmouth Logic code
--

function Loudmouth:Auto_Vars()
  self.current_event_id=self.db.profile["_QuoteEvent"]
  self.current_event_name=self.dropdowns.Lists["_QuoteEvent"].table[self.db.profile["_QuoteEvent"]]
  self.current_set_name=self.dropdowns.Lists["_QuoteSet"].table[self.db.profile["_QuoteSet"]]
  self:Get_Quote_Event(self.current_event_name,self.current_set_name)
  self.current_num_quotes=#quotelist
  self.current_total_quotes = self:Num_Quotes()
end

function Loudmouth:BeatTheOdds(rate)
  local randomvalue = math.random()

  --self:Debug("Random: "..randomvalue.." rate: "..rate)
  if (randomvalue <= (rate/100)) then
    return true
  else
    return false
  end
end

function Loudmouth:Choose_Quote_Event(event, set)
  local index, item

  if not set then
    set = self.db.profile.ActiveSet
  end
  self:Get_Quote_Event(event,set)
  if (#quotelist >= 1) then
    index = math.random(#quotelist)
    while (#quotelist > 1 and self.lastQuoteID[event] == index) do
      index = math.random(#quotelist)
    end
    for item in pairs(quotelist[index]) do
      currentquote[item] = quotelist[index][item]
    end
    self.lastQuoteID[event] = index
  end
end

function Loudmouth:Get_Event_State(event)
  if not event then
    event = self.current_event_name
  end

  return self.db.profile.enabled[event]
end

function Loudmouth:Get_Quote_Event(event,set)
  local all, index, item, offset
  local allquotes = self.db.profile.quotes
  local myquotes = self.db.char.quotes
  
  all = false
  offset = 0
  self:Reset_Quote_Vars()

  if (not set or set == "All") then
    set = self.current_set_name
    all = true
  end

  if (set ~= "Class" and set ~= "Char") then
    for index = 1, #self.db.profile.quotes[set][event]
	 do
		quotelist[index] = {}
      quotelist[index]["quote"] = allquotes[set][event][index]["quote"]
	   quotelist[index]["emote"] = allquotes[set][event][index]["emote"]
      offset = index
	 end
  end
  if (all or set == "Char") then
    for index = 1, #self.db.char.quotes[event]
	 do
		offset = offset + 1
		quotelist[offset] = {}
	   quotelist[offset]["quote"] = myquotes[event][index]["quote"]
		quotelist[offset]["emote"] = myquotes[event][index]["emote"]
	 end
  end
  if (self.db.profile.Debug) then
    for index = 1, #quotelist
    do
  	   self:Debug("#"..index.." '"..quotelist[index]["quote"].."' Emote: "..quotelist[index]["emote"])
    end
  end
end

function Loudmouth:Get_Rate(event)
  if not event then
  event = self.current_event_name
  end

  return self.db.profile.rates[event]
end

function Loudmouth:Num_Quotes()
  local number = 0
  local setidx = ""
  local eventidx = ""

  setidx = next(self.db.profile.quotes, nil)
  while (setidx ~= nil)
  do
  	 eventidx = next(self.db.profile.quotes[setidx], nil)
	 while (eventidx ~= nil)
	 do
	   self:Debug("Number of quotes in "..setidx.."-"..eventidx.." "..#self.db.profile.quotes[setidx][eventidx])
		number = number + #self.db.profile.quotes[setidx][eventidx]
	 	eventidx = next(self.db.profile.quotes[setidx], eventidx)
	 end
	 setidx = next(self.db.profile.quotes, setidx)
  end
  eventidx = next(self.db.char.quotes, nil)
  while (eventidx ~= nil)
  do
	 self:Debug("Number of quotes in "..eventidx.." "..#self.db.char.quotes[eventidx])
	 number = number + #self.db.char.quotes[eventidx]
	 eventidx = next(self.db.char.quotes, eventidx)
  end
  return number
end

function Loudmouth:Output_AFK()
  if (self:Get_Event_State("AFK")) then
    self:Choose_Quote_Event("AFK")
    SendChatMessage(currentquote["quote"],"AFK")
    SendChatMessage(currentquote["quote"],"Guild")
    if (GetNumRaidMembers() > 1) then
      SendChatMessage(currentquote["quote"],"Raid")
    elseif (GetNumPartyMembers() >= 1) then
      SendChatMessage(currentquote["quote"],"Party")
    end
  end
end

function Loudmouth:Output_Excuse()
  if (self:Get_Event_State("AFK")) then
    self:Choose_Quote_Event("Excuses")
    self:Spew()
  end
end

function Loudmouth:Quote_Delete()
  local idx = self.editing_item
  local i

  table.remove(self.db.profile.quotes[self.current_set_name][self.current_event_name],self.editing_item)
  self:GUI_Input_Disable()
   LMFrame_QuoteEdit:ClearFocus()
  self:Auto_Vars()
  self:Update()
  self:GUI_ListBox_OnShow("LMFrame_QuoteList")
end

function Loudmouth:Quote_Add(quote, set, event)
  local newindex
  if (set == "Char") then
    newindex = #self.db.char.quotes[event] + 1
    self.db.char.quotes[event][newindex] = {}
	 self.db.char.quotes[event][newindex]["quote"] = quote
	 self.db.char.quotes[event][newindex]["emote"] = LMEditFrame_Emote:GetChecked()
  else
    newindex = #self.db.profile.quotes[set][event] + 1
    self.db.profile.quotes[set][event][newindex] = {}
	 self.db.profile.quotes[set][event][newindex]["quote"] = quote
	 self.db.profile.quotes[set][event][newindex]["emote"] = LMEditFrame_Emote:GetChecked()
  end
  self:GUI_ListBox_OnShow("LMFrame_QuoteList")
  self:Auto_Vars()
  self:GUI_Input_Disable(1)
  local _,max=LMFrame_QuoteList_ScrollFrame_ScrollBar:GetMinMaxValues()
  LMFrame_QuoteList_ScrollFrame_ScrollBar:SetValue(max)
  self:GUI_ListBox_OnScroll()
end

function Loudmouth:Quote_Edit()
  self:Auto_Vars()
  local quote = LMFrame_QuoteEdit:GetText()
  --local event = self.db.profile["_QuoteEvent"]
  --local event_name = self.dropdowns.Lists["_QuoteEvent"].table[event]
  --local next_index = #self.db.profile.quotes[self.current_set_name][self.current_event_name]+1
  local last_seen = 0

  if (strlen(quote) == 0) then	
    --self:Print(L["Quote was blank. Skipping."])
  	 return
  end
  last_seen = self.editing_item
  if (self.editing_item == 0) then
    self:Quote_Add(quote,self.current_set_name,self.current_event_name)
	 return
  --end
  --if (self.editing_item > 0) then
  else
    self:Quote_Replace(quote,self.current_set_name,self.current_event_name)
	 return
  end

  if (self.editing_item > 0 and strlen(quote) > 0) then	
	if (not self.db.profile.quotes[self.current_set_name][self.current_event_name][self.editing_item]) then
		self.db.profile.quotes[self.current_set_name][self.current_event_name][self.editing_item] = {}
	end
  	self.db.profile.quotes[self.current_set_name][self.current_event_name][self.editing_item]["quote"] = quote
	self.db.profile.quotes[self.current_set_name][self.current_event_name][self.editing_item]["emote"] = LMEditFrame_Emote:GetChecked()
	self.editing_item = 0
  end

  self:GUI_ListBox_OnShow("LMFrame_QuoteList")
  self:GUI_Input_Disable()
   LMFrame_QuoteEdit:ClearFocus()
  self:Auto_Vars()
  if (scrolltobottom)
  then	local _,max=LMFrame_QuoteList_ScrollFrame_ScrollBar:GetMinMaxValues()
  			LMFrame_QuoteList_ScrollFrame_ScrollBar:SetValue(max)
  			self:GUI_ListBox_OnScroll()
  end
end

function Loudmouth:Quote_Has_Options(number)
  local result = false

  if (quotelist[number]["emote"] == 1) then
    result = true
  end
  return result
end

function Loudmouth:Quote_Replace(quote, set, event)
  if (set == "Char") then
	 self.db.char.quotes[event][self.editing_item]["quote"] = quote
	 self.db.char.quotes[event][self.editing_item]["emote"] = LMEditFrame_Emote:GetChecked()
  else
	 self.db.profile.quotes[set][event][self.editing_item]["quote"] = quote
	 self.db.profile.quotes[set][event][self.editing_item]["emote"] = LMEditFrame_Emote:GetChecked()
  end
  self:GUI_ListBox_OnShow("LMFrame_QuoteList")
  self:Auto_Vars()
  self:GUI_Input_Disable(1)
  LMFrame_QuoteEdit:ClearFocus()
end

function Loudmouth:Reset_Quote_Vars()
  local item = nil

  statement = ""
  currentquote["quote"] = ""
  currentquote["emote"] = 0

  for item = 1, #quotelist
  do
    table.remove(quotelist,item)
  end
end

function Loudmouth:Set_Event_State(event,value)
  self.db.profile.enabled[event] = value
end

function Loudmouth:Set_Rate(event, value)
  self.db.profile.rates[event] = value
end

function Loudmouth:Spew()
  if (statement == '') then
    if currentquote["quote"] == '' then 
	   return 
    else
	   statement = currentquote["quote"]
	 end
  end

  local output = self.db.profile.Channel
  local orig_output = output
  
  self.LastQuoted = GetTime()
  if (GetNumRaidMembers() == 0 and output == 'Raid') then output = 'Party'; end 
  if (GetNumPartyMembers() == 0 and output == 'Party') then output = 'Say'; end

  if (orig_output ~= output) 
  then self:Debug("Downgrading from "..orig_output.." to "..output)
  end

  if (currentquote["emote"] == 1) then output = "EMOTE" end

  if (output == "UI") then
    self:Print(statement)
  else
    SendChatMessage(statement,output)
  end

  --
  -- The following line probably doesn't belong here, I just wanted to save 
  -- this default in a function I'm sure will be called eventually
  --
  self.db.profile.Debug = self:IsDebugging()
end

function Loudmouth:Throttle()
  if (self.db.profile.Debug) then
    return false
  end

  if (GetTime() - self.LastQuoted < self.db.profile.Throttle) then	
    self:Debug("Throttling output, said something too recently.")
  	 return true
  end
  return false
end

--
-- All Event handlers are located below this comment
--

function Loudmouth:RegisterMyEvents()
  self:RegisterEvent("CHAT_MSG_COMBAT_SELF_HITS") -- arg1=msg
  self:RegisterEvent("CHAT_MSG_COMBAT_SELF_MISSES") -- arg1=msg
  self:RegisterEvent("CHAT_MSG_EMOTE") -- arg1=msg arg2=player
  self:RegisterEvent("CHAT_MSG_MONSTER_EMOTE") -- arg1=msg
  self:RegisterEvent("CHAT_MSG_SYSTEM")
  self:RegisterEvent("CHAT_MSG_TEXT_EMOTE") -- arg1=msg arg2=player
  --self:RegisterEvent("CHAT_MSG_BG_SYSTEM_ALLIANCE") -- arg1
  --self:RegisterEvent("CHAT_MSG_BG_SYSTEM_HORDE") -- arg1
  --self:RegisterEvent("CHAT_MSG_SKILL") -- arg1=msg
  --self:RegisterEvent("PLAYER_LEVEL_UP") -- arg1=level arg2-9=vals
  --self:RegisterEvent("PLAYER_PVP_KILLS_CHANGED")
  --self:RegisterEvent("PLAYER_TARGET_CHANGED")
    --local name,realm = UnitName("target")
  --self:RegisterEvent("SPELLCAST_CHANNEL_START") -- arg1=ms arg2=spell
  --self:RegisterEvent("SPELLCAST_CHANNEL_STOP")
  --self:RegisterEvent("SPELLCAST_START") --arg1=spell arg2=ms
  --self:RegisterEvent("SPELLCAST_STOP")
end

function Loudmouth:CHAT_MSG_COMBAT_SELF_HITS()
  if (self:Throttle()) then return; end
  local you1, you2 = string.find(arg1, L["You"]);
  local crit, crit2 = string.find(arg1, L["crit"]);

  self:Reset_Quote_Vars()
  if (self.db.profile.enabled["Crits"]) then
    if (you1 == 1 and you2 == 3 and crit) then
      if (self:BeatTheOdds(self.db.profile.rates["Crits"])) then
		  self:Choose_Quote_Event("Crits")
      end
    end
  end
  self:Spew()
end

function Loudmouth:CHAT_MSG_COMBAT_SELF_MISSES()
  if (self:Throttle()) then return; end
  local message = "Loudmouth: miss"
  local item

  self:Reset_Quote_Vars()

  if (self.db.profile.enabled["Misses"]) then
    if (self:BeatTheOdds(Loudmouth.db.profile.rates["Misses"])) then
	   self:Choose_Quote_Event("Misses")
    end
  end
  self:Spew()
end

function Loudmouth:CHAT_MSG_EMOTE()
  --self:Debug("CHAT_MSG_EMOTE: "..arg1.." "..arg2);
end

function Loudmouth:CHAT_MSG_MONSTER_EMOTE()
  if (self:Throttle()) then return; end
  local message = "Loudmouth: runaway "..arg2
  local you1, you2 = string.find(arg1, L["run"]);
  local fear, fear2 = string.find(arg1, L["fear"]);

  self:Reset_Quote_Vars()
  if (you1 ~= nil and fear ~= nil and self.db.profile.enabled["Fears"]) then
    if (self:BeatTheOdds(self.db.profile.rates["Fears"])) then
	   self:Choose_Quote_Event("Misses")
    end
  end
  self:Spew()
end

function Loudmouth:CHAT_MSG_SYSTEM()
  if not string.find(arg1,L["You are now AFK"]) then return end
  --self:Debug(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9)
  self:UnregisterEvent("CHAT_MSG_SYSTEM")
  Loudmouth:Output_AFK()
  self:RegisterEvent("CHAT_MSG_SYSTEM")
end

function Loudmouth:CHAT_MSG_TEXT_EMOTE()
  self:Debug("CHAT_MSG_TEXT_EMOTE: "..arg1.." "..arg2);
end

--
-- Code Snippets and Comments I want to remember are here to the end. 
-- Probably nothing of real interest down here. :)
--

  --UIErrorsFrame:AddMessage(message,1.0,1.0,1.0,5.0)
  --self:Debug(message)
--LMF = CreateFrame("Frame",nil,UIParent)
--LMFt=LMF:CreateTexture(nil,"BACKGROUND")
--LMFt:SetTexture("Interface\\DialogFrame\\UI-DialogBox-Background")
--LMFt:SetAllPoints(LMF)
--LMF.texture=LMFt
--shown = false

  --LMF:SetFrameStrata("BACKGROUND")
  --LMF:SetWidth(100)
  --LMF:SetHeight(100)
  --LMF:SetPoint("CENTER",0,0)
  --LMF:SetMovable(true)
  --if (shown == true) then
    --LMF:Hide()
    --shown = false
  --else
    --LMF:Show()
    --shown = true
  --end

-- UnitSex("Unit") -> 1 = unknown, 2 = male, 3 = female
--function SU_EditBox:SetText(whatCommand)
  --local editBox = SU_EditBox;
  -- send the slash command to the chat box
  --ChatEdit_SendText(editBox);
--end
-- vim: ts=3 ai nu
